<?php

/**
 * Tell Drupal that we have an additional validation function.
 */
function quadstat_core_form_alter(array &$form, \Drupal\Core\Form\FormStateInterface $form_state, $form_id) {
  if(strpos($form_id, 'webform_submission_') !== FALSE) {
    $form['actions']['submit']['#validate'][] = 'quadstat_core_application_form_validate';
  }
}

/**
 * Execute the command from the webform
 */
function quadstat_core_application_form_validate(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
  // Get the values the user submitted
  $user = \Drupal::currentUser();
  $form_input = $form_state->getValues();
  $dataset_setup_cmd = _quadstat_core_setup_dataset($form, $form_state, $form_input);
  if(!isset($form_input['r'])) {
    // This is not an R application
    return;
  }
  $command = $form_input['r'];
  // Replace each variable with the user input
  foreach($form_input as $key => $input) {
    $command = str_replace('$' . $key, $form_input[$key], $command);
  }
  $identifier = _quadstat_core_get_identifier();
  if(array_key_exists('image', $form_input)) {
    // User wants an image
    $img_setup = _quadstat_core_setup_img($form, $form_state, $identifier);
    // $img_save has dev.off()
    $img_save = _quadstat_core_save_img($form, $form_state, $identifier);
  } else {
    // No image will be generated
    $img_setup = $img_save = '';
  }
  $command = $dataset_setup_cmd . $img_setup . $command .$img_save;
  _quadstat_core_execute_r($command, $identifier, TRUE);
  $tmp_path = \Drupal::service('file_system')->realpath('temporary://');
  // Let user know of any errors or warnings while processing dataset request
  if(is_file($error_file = "$tmp_path/application-$identifier.err") && filesize($error_file) > 0) {
    drupal_set_message(file_get_contents($error_file), 'error');
  }
  if(is_file($warn_file = "$tmp_path/application-$identifier.warn") && filesize($warn_file) > 0) {
    drupal_set_message(file_get_contents($warn_file), 'warning');
  }
  $output_file = "$tmp_path/application-$identifier.Rout";
  $output = file_get_contents($output_file);
  $form_input['r'] = $command;
  $form_input['output'] = $output;
  if(is_file("$tmp_path/application-$identifier.png")) {
    // Now save the file to the Drupal database
    $data = file_get_contents("$tmp_path/application-$identifier.png");
    $username = $user->getAccountName();
    $png_dir = "private://webform/plot/$username";
    \Drupal::service('file_system')->mkdir($png_dir);
    if (file_prepare_directory($png_dir)) {
      $file = file_save_data($data, "$png_dir/application-$identifier.png", FILE_EXISTS_RENAME);
      if($file) {
        $form_input['image'] = $file->get('fid')->get(0)->value;
      }
    }
  }
  $form_state->setValues($form_input);
}

function _quadstat_core_setup_dataset(&$form, $form_state, $form_input) {
  $fid = isset($form_input['x']) ? $form_input['x'] : NULL;
  $v1 = isset($form_input['v1']) ? $form_input['v1'] : NULL;
  $v2 = isset($form_input['v2']) ? $form_input['v2'] : NULL;
  $v3 = isset($form_input['v3']) ? $form_input['v3'] : NULL;
  if(is_null($fid)) {
    // A dataset is not involved; do nothing
    return '';
  }
  $user = \Drupal::currentUser();
  $file = file_load($fid);
  $uri = $file->get('uri')->get(0)->value;
  $filepath = \Drupal::service('file_system')->realpath($uri); 
  $command = '';
  $command .= "options(echo = FALSE);\n";
  // Uncomment the libpath variable for custom libraries
  // $command .= ".libPaths(\"/home/www-data/R/x86_64-pc-linux-gnu-library/3.0\");\n";
  $command .= "X <- read.table(\"$filepath\", sep=\",\", header = TRUE);\n";
  if($v1 != '-1' && $v2 != '-1'  && $v3 != '-1') {
    // The user has chosen 3 vectors
    $command .= "colNam <- colnames(X);\n";
    $command .= "X <- cbind(X[,$v1], X[,$v2], X[,$v3]);\n";
    $command .= "colnames(X) <- c(colNam[$v1], colNam[$v2], colNam[$v3]);\n";
  } else if($v1 != '-1' && $v2 != '-1') {
    // The user has chosen 2 vectors
    $command .= "colNam <- colnames(X);\n";
    $command .= "X <- cbind(X[,$v1], X[,$v2]);\n";
    $command .= "colnames(X) <- c(colNam[$v1], colNam[$v2]);\n";
  }
  else if($v1 != '-1') {
    // The user has chosen 1 vector
    $command .= "colNam <- colnames(X);\n";
    $command .= "X <- cbind(X[,$v1]);\n";
    $command .= "colnames(X) <- c(colNam[$v1]);\n";
  } else {
    // The user has not selected any vectors, use the whole dataset. Don't need to do anything.
  }
  return $command;
}

function _quadstat_core_setup_img(&$form, $form_state, $identifier) {
  $tmp_path = \Drupal::service('file_system')->realpath('temporary://');
  $setup_img_cmd = "png(file = \"$tmp_path/application-$identifier.png\", bg = \"transparent\");\n";
  return $setup_img_cmd;
}

function _quadstat_core_save_img(&$form, $form_state, $identifier) {
  return $save_img_cmd = "dev.off();\n";
}
