/**
 * @file
 * Javascript behaviors for viewing data in the editor
 */

/* global Quadstat_Core, Drupal, jQuery, document */

(function ($, Drupal, document, Quadstat_Core) {

  'use strict';

  /**
   * Attaches behaviors for Quadstat.
   */
  Drupal.behaviors.quadstat_core = {
    attach: function (context, settings) {

      // Hide dataset block if .no-dataset exists on page (set through Webform UI CSS)
      if($('.no-dataset').length) {
        $('#block-views-block-dataset-block-1, #quadstat-slickgrid').css('display', 'none');
      }
      // Function to highlight columns upon click
      function highlight(col) {
        if($('div.slick-header-column:eq(' + col + ')').hasClass('highlighted-col')) {
          // column already highlighted, deselect
          $('.slick-cell.r' + col).removeClass('highlighted-col-1');
          $('.slick-header-column:eq(' + col + ')').removeClass('highlighted-col');
          $('.slick-header-column:eq(' + col + ')').removeClass('highlighted-col-1');
          $('.slick-cell.r' + col).removeClass('highlighted-col-2');
          $('.slick-header-column:eq(' + col + ')').removeClass('highlighted-col-2');
          $('.slick-header-column:eq(' + col + ')').removeClass('highlighted-col-3');
          $('.slick-cell.r' + col).removeClass('highlighted-col-3');
          return true;
        }
        if($('div.slick-header-column.highlighted-col').length == 0) {
          // no highlighted column, select first one
          $('.slick-cell.r' + col).addClass('highlighted-col');
          $('.slick-cell.r' + col).addClass('highlighted-col-1');
          $('.slick-header-column:eq(' + col + ')').addClass('highlighted-col');
          $('.slick-header-column:eq(' + col + ')').addClass('highlighted-col-1');
          return true;
        }
        if($('div.slick-header-column.highlighted-col').length == 2) {
          if($('.slick-header-column').hasClass('highlighted-col-1') && $('.slick-header-column').hasClass('highlighted-col-2')) {
            // highlight 3rd column
            $('.slick-cell.r' + col).addClass('highlighted-col-3');
            $('.slick-cell.r' + col).addClass('highlighted-col');
            $('.slick-header-column:eq(' + col + ')').addClass('highlighted-col');
            $('.slick-header-column:eq(' + col + ')').addClass('highlighted-col-3');
            return true;
          }
          if($('.slick-header-column').hasClass('highlighted-col-2') && $('.slick-header-column').hasClass('highlighted-col-3')) {
            // highlight 1st column
            $('.slick-cell.r' + col).addClass('highlighted-col-1');
            $('.slick-cell.r' + col).addClass('highlighted-col');
            $('.slick-header-column:eq(' + col + ')').addClass('highlighted-col');
            $('.slick-header-column:eq(' + col + ')').addClass('highlighted-col-1');
            return true;
          }
          if($('.slick-header-column').hasClass('highlighted-col-1') && $('.slick-header-column').hasClass('highlighted-col-3')) {
            // highlight 2nd column
            $('.slick-cell.r' + col).addClass('highlighted-col-2');
            $('.slick-cell.r' + col).addClass('highlighted-col');
            $('.slick-header-column:eq(' + col + ')').addClass('highlighted-col');
            $('.slick-header-column:eq(' + col + ')').addClass('highlighted-col-2');
            return true;
          }
        }
        if($('div.slick-header-column.highlighted-col').length == 3) {
          // already got three, no mas!
          return true;
        }
        if($('div.slick-header-column.highlighted-col').length == 1) {
          // got one column, let's highlight a second one
          if($('.slick-header-column').hasClass('highlighted-col-1') && !$('.slick-header-column').hasClass('highlighted-col-3')) {
            // no 2nd column
            $('.slick-cell.r' + col).addClass('highlighted-col-2');
            $('.slick-cell.r' + col).addClass('highlighted-col');
            $('.slick-header-column:eq(' + col + ')').addClass('highlighted-col');
            $('.slick-header-column:eq(' + col + ')').addClass('highlighted-col-2');
            return true;
          }
          if($('.slick-header-column').hasClass('highlighted-col-2') && !$('.slick-header-column').hasClass('highlighted-col-1')) {
            // no 3rd column
            $('.slick-cell.r' + col).addClass('highlighted-col-1');
            $('.slick-cell.r' + col).addClass('highlighted-col');
            $('.slick-header-column:eq(' + col + ')').addClass('highlighted-col');
            $('.slick-header-column:eq(' + col + ')').addClass('highlighted-col-1');
            return true;
          }
          if($('.slick-header-column').hasClass('highlighted-col-3') && !$('.slick-header-column').hasClass('highlighted-col-2')) {
            // no 1st column
            $('.slick-cell.r' + col).addClass('highlighted-col-2');
            $('.slick-cell.r' + col).addClass('highlighted-col');
            $('.slick-header-column:eq(' + col + ')').addClass('highlighted-col');
            $('.slick-header-column:eq(' + col + ')').addClass('highlighted-col-2');
            return true;
          }
        }
      }

      /*
       * The rest of the code in this file is for Slickgrid preview
       */

      var grid;
      var options;
      var data = [];
      var d = [];
      var columns = [];

      // Define options for grid
      var options = {
        editable: false,
        enableAddRow: false,
        enableCellNavigation: false,
        forceFitColumns: false,
        enableColumnReorder: false,
        showHeaderRow: false,
        autoedit: false,
      };

      // Setup columns
      for(var i = 0; i < drupalSettings.quadstat_core.data.length; i++) {
        columns[i] = [{id:0, name:"Rows", field:0, cssClass:"cell-title", formatter: function(row){return row+1}}];
        for(var j = 0; j < drupalSettings.quadstat_core.data[i]['longest_row']; j++) {
          columns[i].push({id: j, name: drupalSettings.quadstat_core.data[i]['val'][0][j], field:j});
        }
      }

      function setItems(data, objectIdProperty) {
        if (objectIdProperty !== undefined) {
          idProperty = objectIdProperty;
        }
        items = data;
        refreshIdxById();
        refresh();
      }

      // Setup rows, minimum 10x10 grid
      for (var i = 0; i < drupalSettings.quadstat_core.data.length; i++ ) {
        data[i] = new Array();
        for (var j = 1; j < drupalSettings.quadstat_core.data[i]['val'].length; j++) {
          data[i][j - 1] = new Array();
          for (var k = 0; k < drupalSettings.quadstat_core.data[i].val[j].length; k++) {
            data[i][j - 1][k]  = drupalSettings.quadstat_core.data[i].val[j][k];
          }
        }
      }

      if($('#quadstat-slickgrid').length) {
        grid = new Slick.Grid("#quadstat-slickgrid", data[0], columns[0], options);
        grid.init();   
      } else {
        return;
      }

      $('#block-views-block-dataset-block-1 table').on('dataset_changed', 'td.views-field.views-field-fid', load_new_dataset);
      function load_new_dataset() { 
        var fid = Number($('td.views-field.views-field-fid').html());
        grid.invalidate();
        grid = new Slick.Grid("#quadstat-slickgrid", data[drupalSettings.quadstat_core.map[fid]], columns[drupalSettings.quadstat_core.map[fid]], options);
        grid.init();
        // Tell Quadstat which dataset to perform the operation on
        $('#edit-x').val(fid);
        // Clicking a header columns triggers the column highlighter and records selected dataset input column vectors
        $('.slick-header-column').click(function() {
          var col = $(this).index();
          if($('#quadstat-slickgrid .highlighted-col-1').length === 0) {
            $('#edit-v1').val(col);
            $('#edit-v2').val('-1');
            $('#edit-v3').val('-1');
          } else if ($('#quadstat-slickgrid .highlighted-col-2').length === 0 && $('#edit-v1').val() != col) {
            $('#edit-v2').val(col);
            $('#edit-v3').val('-1');
          } else if ($('#quadstat-slickgrid .highlighted-col-3').length === 0 && $('#edit-v2').val() != col) {
            $('#edit-v3').val(col);
          }
          highlight(col);
        });
      }

      $('#block-views-block-dataset-block-1 td.views-field.views-field-fid').trigger('dataset_changed');

      $('#block-views-block-dataset-block-1 #edit-title').val($('#block-views-block-dataset-block-1 .views-field-title').html().trim());

    }
  };
}(jQuery, Drupal, document));
